from .sylia import Clock as clock, Screen as screen, Duck as duck, init
from .sylia import Sound as sound, Key as key, Mouse as mouse, running

from .image import Image as image
from .shape import Shape as shape
from .text import Text as text